import os

from .model_api import model_api

api_key = os.environ.get("API_KEY", "mega")

api_timeout_seconds = 300

model_api_url = os.environ.get(
    "MEGATECHAI_MODEL_API_URL", "http://192.168.1.188:3004/model-api"#"http://region-31.seetacloud.com:39638/api/chat-process"
)



# -*- coding:utf-8 -*-
import posixpath

import megatechai
from megatechai.utils.http_client import get, post, stream
from megatechai.utils.sse_client import SSEClient


class InvokeType:
    SYNC = "invoke"
    ASYNC = "async-invoke"
    SSE = "sse-invoke"


class ModelAPI:

    @classmethod
    def model_list(cls):
        url = megatechai.model_api_url + "/model-list"
        try:
            models = get(url, cls._generate_token(), megatechai.api_timeout_seconds).get("data", [])
        except:
            models = []
        return models

    @classmethod
    def invoke(cls, **kwargs):
        url = cls._build_api_url(kwargs, InvokeType.SYNC)
        return post(url, cls._generate_token(), kwargs, megatechai.api_timeout_seconds)

    @classmethod
    def async_invoke(cls, **kwargs):
        url = cls._build_api_url(kwargs, InvokeType.ASYNC)
        return post(url, cls._generate_token(), kwargs, megatechai.api_timeout_seconds)

    @classmethod
    def query_async_invoke_result(cls, task_id: str):
        url = cls._build_api_url(None, InvokeType.ASYNC, task_id)
        return get(url, cls._generate_token(), megatechai.api_timeout_seconds)

    @classmethod
    def sse_invoke(cls, **kwargs):
        url = cls._build_api_url(kwargs, InvokeType.SSE)
        data = stream(url, cls._generate_token(), kwargs, megatechai.api_timeout_seconds)
        return SSEClient(data)

    @staticmethod
    def _build_api_url(kwargs, *path):
        if kwargs:
            if "model" not in kwargs:
                raise Exception("model param missed")
        return megatechai.model_api_url + "/model-api"
        #return posixpath.join(megatechai.model_api_url, model, *path)

    @staticmethod
    def _generate_token():
        if not megatechai.api_key:
            raise Exception(
                "api_key not provided, you could provide it with `shell: export API_KEY=xxx` or `code: megatechai.api_key=xxx`"
            )

        return megatechai.api_key



# -*- coding:utf-8 -*-
import time

import cachetools.func
import jwt

API_TOKEN_TTL_SECONDS = 3 * 60

CACHE_TTL_SECONDS = API_TOKEN_TTL_SECONDS - 30


@cachetools.func.ttl_cache(maxsize=10, ttl=CACHE_TTL_SECONDS)
def generate_token(apikey: str):
    try:
        api_key, secret = apikey.split(".")
    except Exception as e:
        raise Exception("invalid api_key", e)

    payload = {
        "api_key": api_key,
        "exp": int(round(time.time() * 1000)) + API_TOKEN_TTL_SECONDS * 1000,
        "timestamp": int(round(time.time() * 1000)),
    }

    return jwt.encode(
        payload,
        secret,
        algorithm="HS256",
        headers={"alg": "HS256", "sign_type": "SIGN"},
    )



# -*- coding:utf-8 -*-
import json
import logging

import requests

headers = {
    "Accept": "application/json",
    "Content-Type": "application/json; charset=UTF-8",
}


def post(api_url, token, params, timeout):
    try:
        headers.update({"token": token})
        resp = requests.post(
            url=api_url, data=json.dumps(params), headers=headers, timeout=timeout
        )
        if requests.codes.ok != resp.status_code:
            raise Exception("响应异常：" + resp.content)
        return json.loads(resp.text)
    except Exception as e:
        logging.exception("请求异常", e)


def stream(api_url, token, params, timeout):
    try:
        resp = requests.post(
            api_url,
            stream=True,
            headers={"token": token},
            json=params,
            timeout=timeout,
        )
        if requests.codes.ok != resp.status_code:
            raise Exception("请求异常")
        return resp
    except Exception as e:
        logging.exception("请求异常", e)


def get(api_url, token, timeout):
    try:
        headers.update({"token": token})
        resp = requests.get(api_url, headers=headers, timeout=timeout)
        if requests.codes.ok != resp.status_code:
            raise Exception("响应异常：" + resp.content)
        return json.loads(resp.text)
    except Exception as e:
        logging.exception("请求异常", e)
